/* 
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#ifndef BACKEND_H
#define BACKEND_H

#include "QtCore/qcontainerfwd.h"
#include "QtCore/qtmetamacros.h"
#include "QtCore/qvariant.h"
#include <QtQml/qqmlregistration.h>
#include <QtCore/QObject>
#include <QtCore/QDateTime>
#include <QtCore/QString>
#include <QtCore/QList>
#include <QtCore/QDir>
#include <QtCore/QJsonObject>
#include <QtCore/QJsonDocument>
#include <QtCore/QJsonArray>
#include <iostream>
#include <memory>
#include "core/TaskItem.h"
#include "core/TasksView.h"
#include "core/TodoMd.h"
#include "core/Mirai.h"

#include "TaskItem.h"

class Backend : public QObject
{
    Q_OBJECT
    QML_ELEMENT
    Q_PROPERTY(QVariant tasks READ getTasks NOTIFY tasksChanged)
    Q_PROPERTY(QVariant tags READ getTags NOTIFY tasksChanged)
    Q_PROPERTY(QVariant activeTagsFilter READ getActiveTagsFilter NOTIFY tasksChanged)
	Q_PROPERTY(bool shouldHideCompletedTasks READ shouldHideCompletedTasks WRITE hideCompletedTasks NOTIFY tasksChanged)

public:

	Backend();

	Q_INVOKABLE void addTodo(QString newTodo, QString date);
	Q_INVOKABLE void addTodoFromRawFormat(QString text, QString date);
	Q_INVOKABLE void addTagFilter(QString tag);
	Q_INVOKABLE void removeTagFilter(QString tag);
	Q_INVOKABLE void removeFilters();
	Q_INVOKABLE void updateTodoFromRawFormat(int todoIndex, QString text, QString date);
	Q_INVOKABLE void updateTodo(int todoIndex, QString state, QString text, QString date);
	Q_INVOKABLE void removeTodo(int todoIndex);
	Q_INVOKABLE void hideCompletedTasks(bool shouldHide);

private:

	void rebuildQMLTasksList();

	QVariant getTasks();
	QVariant getTags();
	QVariant getActiveTagsFilter();
	bool shouldHideCompletedTasks();

	mirai::Mirai mirai;
	std::weak_ptr<mirai::TasksView> view;

	QList<QMLTaskItem> QMLTasks;
	QList<QString> QMLTags;
	QList<QString> QMLActiveTagsFilter;

	bool shouldHideCompletedTasks_ = true;

signals: 

	void tasksChanged();
};

#endif
